const PccApp = require("../../../picockpit-library/picockpit-base-app")
const pythonPath = "../../build/python_binaries/python3.11"
const { spawn } = require('child_process')


class Bme688Airmeter extends PccApp{
    constructor(){
        super("bme688airmeter")
        this.logger.info('Constructing bme688airmeter')
        this.publishCategories()

        this.lastMeasurment
        this.handleSubprocess()
        this.measuringLoop()

    }

    measuringLoop = async () => {
        while (true) {
            await this.sleep(5000)
            this.logger.debug("Publishing data")
            // Only if not error code
            this.publish("$state", JSON.parse(this.lastMeasurment))
        }
    }


    handleSubprocess = async () => {
        // If I2C isn't enabled this will throw an error
        
        const process = spawn(
            `
            source ../picockpit-apps/com.picockpit/bme688airmeter/.venv/bin/activate
            ../../../../../build/python_binaries/python3.11 ../picockpit-apps/com.picockpit/bme688airmeter/readSensor.py        
            `, 
            { shell: "/bin/bash"}
        )

        process.stdout.on("data", (data) => {
            // Add error handling for wrong output
            this.logger.debug("Received data from sensor")
            this.logger.debug(data)
            this.logger.debug(data.toString())
            // Problem is that paths have changed after merge
            this.lastMeasurment = (data.toString()).replaceAll("'", '"')
        })
        
        process.on("error", (error) => {
            this.logger.error("Process crashed because of following error: ", error)
        })

        process.on("exit", (code) => {
            this.logger.verbose("Process exited with code: ", code)
            this.lastMeasurment = { "status": code }
        })
    }

    publishCategories = async () => {
        const meta = {
            "sample_nr": {
                "$description": "sample number",
                "$unit": ""
            },
            "timestamp": {
                "$description": "timestamp",
                "$unit": "ms"
            },
            "raw_temperature": {
                "$description": "air temperature (raw)",
                "$unit": "°C",
                "$min": -40,
                "$max": 85
            },
            "raw_pressure": {
                "$description": "air pressure",
                "$unit": "hPa",
                "$min": 300,
                "$max": 1100
            },
            "raw_humidity": {
                "$description": "air humidity (raw)",
                "$unit": "%",
                "$min": 0,
                "$max": 100
            },
            "raw_gas": {
                "$description": "gas resistance (raw)",
                "$unit": "kΩ"
            },
            "status": {
                "$description": "status code",
                "$unit": ""
            },
            "iaq": {
                "$description": "air quality (IAQ)",
                "$unit": ""
            },
            "iaq_accuracy": {
                "$description": "air quality (IAQ) accuracy",
                "$unit": ""
            },
            "temperature": {
                "$description": "air temperature",
                "$unit": "°C",
                "$min": -40,
                "$max": 85
            },
            "humidity": {
                "$description": "air humidity",
                "$unit": "%",
                "$min": 0,
                "$max": 100
            },
            "static_iaq": {
                "$description": "static IAQ (raw)",
                "$unit": ""
            },
            "static_iaq_accuracy": {
                "$description": "static IAQ accuracy",
                "$unit": ""
            },
            "co2_equivalent": {
                "$description": "CO2 equivalent",
                "$unit": ""
            },
            "co2_accuracy": {
                "$description": "CO2 equivalent accuracy",
                "$unit": ""
            },
            "breath_voc_equivalent": {
                "$description": "breath VOC equivalent",
                "$unit": ""
            },
            "breath_voc_accuracy": {
                "$description": "breath VOC equivalent accuracy",
                "$unit": ""
            },
            "comp_gas_value": {
                "$description": "comp gas value",
                "$unit": ""
            },
            "comp_gas_accuracy": {
                "$description": "comp gas value accuracy",
                "$unit": ""
            },
            "gas_percentage": {
                "$description": "gas percentage",
                "$unit": ""
            },
            "gas_percentage_accuracy": {
                "$description": "gas percentage accuracy",
                "$unit": ""
            },
        }

        await this.sleep(5000)

        this.logger.verbose("Publishing $meta")
        this.publish("$meta", meta, {retain: true})
    }

    sleep = async (ms) => {
        return new Promise((resolve) => {
            setTimeout(resolve, ms);
        });
    }

}

bme688airmeter = new Bme688Airmeter()
