const PccApp = require("../../../picockpit-library/picockpit-base-app")
const i2c = require('i2c-bus')
const constants = require('./constants')

const { EE895ADDRESS, I2CREGISTER } = constants

class CO2 extends PccApp {
    constructor() {
        super("co2sensor")
        this.logger.info('Constructing co2sensor')
        this.i2c = i2c.open(1, err => {
            if (err) throw err
        })
        
        this.publishCategories()
        this.measureLoop()
    }

    async measureLoop(){
        while(true){
            await this.sleep(5000)

            this.i2c.readWord(EE895ADDRESS, I2CREGISTER, (err, rawData) => {
                if (err) throw err
                this.logger.debug('RawData: ', rawData)
            })

            this.dataBuffer = Buffer.alloc(8)
            this.i2c.readI2cBlock(EE895ADDRESS, I2CREGISTER, 8, this.dataBuffer, (err, bytesRead, buffer) => {
                
                if (err) throw err

                const co2 = buffer.readUint16BE(0)
                const temperature = buffer.readUint16BE(2) / 100
                const pressure = buffer.readUint16BE(6) / 10

                const msg = {
                    "t": temperature,
                    "p": pressure,
                    "c": co2
                }

                this.logger.debug(msg)
                this.publish('$state', msg)
            })
        }

    }
    
    publishCategories = async () => {
        const meta = {
            "t": {
                "$description": "air ttemperature",
                "$unit": "°C",
                "$min": -40,
                "$max": 60
            },
            "p": {
                "$description": "air pressure",
                "$unit": "hPa",
                "$min": 700,
                "$max": 1100
            },
            "c": {
                "$description": "CO2",
                "$unit": "ppm",
                "$min": 0,
                "$max": 5000
            }
        }
        
        await this.sleep(5000)
        
        this.logger.verbose("Publishing $meta")
        this.publish("$meta", meta, {retain: true})
    }
    
    sleep = async (ms) => {
        return new Promise((resolve) => {
            setTimeout(resolve, ms);
        });
    }
}

co2 = new CO2()
