
const { spawnSync, execSync } = require('child_process');

/**
 * exec
 * @description Takes a string which might be a literal or encoded command (shell,fn) and 
 * @param {string} cmd  The string to parse
 * @param {string} lang  Used if it is an i18n object 
 * @param {object} fcnList  An object of functions 
 * @returns {String} the literal, fn rv, or stdout
 */
function exec(cmd, lang="en", fcnList) {
    const def = {stdout:"", stderr:"", status:0}
    let result;

    if (typeof cmd == "string" && /^\(.*\)$/.test(cmd)) // bash str cmd
      try {
        result = {...def, stdout:execSync(cmd, {shell:"/bin/bash", timeout:10000, stdio: 'pipe'})};
      } catch (r){
        result = r; // result of underlying spawn
      }

    if (typeof cmd == "string" && /.*\(\)$/.test(cmd)) // function
      try {
        result = {...def, stdout:fcnList[cmd.slice(0, -2)](cmd, lang)}
      } catch (r){
        result = {...def, stderr:r, status:1}; 
      }

    if (Array.isArray(cmd))  // separated cmd
      result = spawnSync(cmd[0], cmd.slice(1), {timeout:2000});

    if (typeof cmd == "object" && "en" in cmd) {  // lang conditional obj
        if (lang in cmd)
            return exec(cmd[lang], lang, fcnList);
        else {
            return exec(cmd.en, lang, fcnList);
        }
    }

    if (typeof cmd == "string" && ! result)
      result = {...def, stdout:cmd} //  literal
    
    if ( ! result )
      throw "Error: Unparsable command format"

    result.stdout = result.stdout.toString('utf8').trim();
    result.stderr = result.stderr.toString('utf8').trim();
    if (result.status != 0)
        throw result;

    return result
}

/**
   * Wraps the exec function.  Logs the object on an exception.
   *
   * @param {string} val - The string to be execed.
   * @returns {string} The exec stdout.
   */
function execLog (val) {
  // console.log("resolve:", val, isOpt)

  try {
      return exec(val).stdout;
      // if (rv == "0" || rv=="1") // is this a universal?
      //     rv = rv == "0"?1:0;
      
  } catch (error) {
      if (typeof error == "string")
          console.log('Error:', error);
      else {
          console.log('Stdout:', error.stdin);
          console.log('Stderr:', error.stderr);
          console.log('Status:', error.status);
      }
      throw 'Failed exit code';
  }
}

module.exports = {exec, execLog};