const fs = require('fs');
const path = require('path');
const c = require('./const');

/**
 * Retrieves a form by name.
 *
 * @param {string} name - The name of the form.
 * @returns {object|null} The form object
 */
function getForm(name) {
  if (! (/^[^.]*$/.test(name))) // no . allowed in name
    return "";
  const filePath = `${c.schemaPath}/forms/${name}.json`
  return JSON.parse(fs.readFileSync(filePath, 'utf8'));
}

/**
 * Retrieves service definitions.
 *
 * @returns {object|null} The services object
 */
function getServices() {
  const directoryPath = `${c.schemaPath}/services`
  const combinedJsonObject = {};

  try {
    const files = fs.readdirSync(directoryPath);

    files.forEach(file => {
      const filePath = path.join(directoryPath, file);
      if (fs.statSync(filePath).isFile() && path.extname(file) === '.json') {
        try {
          const fileContent = fs.readFileSync(filePath, 'utf8');
          const parsedJson = JSON.parse(fileContent);
          Object.assign(combinedJsonObject, parsedJson);
        } catch (parseError) {
          console.error(`Error parsing ${file}: ${parseError.message}`);
        }
      }
    });

    return combinedJsonObject;
  } catch (err) {
    console.error(`Error reading directory: ${err}`);
    return null;
  }   
};

module.exports = { getServices, getForm };
