const { execSync } = require('child_process');

function parseSources(lines) {
  const cameras = [];
  let currentCamera = null;

  // Parse the output to extract camera information

  for (const line of lines) {

    // Regular expression pattern to match the device number and label
    const pattern = /^(\d+)\s+:\s+(\w+)\s+\[.*$/;

    // Extracting dev and label using match method with the pattern
    const match = line.match(pattern);

    try{
      if (match) {
        currentCamera = {
          type:'mipi',
          dev: parseInt(match[1]),
          label: match[2].trim(),
          options: [],
        };
        cameras.push(currentCamera);
      }
  
      const modeDetails = line.match(/(\d+x\d+) \[(\d+\.\d+) fps/);
      if (modeDetails) {
        const resolution = modeDetails[1];
        const fps = parseFloat(modeDetails[2]);
        currentCamera.options.push({ resolution, fps });
      }
    } catch (err) {
      console.log("Error")
    }

  }


  return cameras
}

function getSources() {
  const lines = execSync('libcamera-hello --list-cameras', { encoding: 'utf8' }).split('\n').filter(Boolean);
  return parseSources(lines);
}

module.exports = {
  getSources
};