const { execSync } = require('child_process');

// v4l2
function getModel (dev) {
  const lines = execSync(`v4l2-ctl --all -d ${dev}`, { encoding: 'utf8' }).split('\n').filter(Boolean);
  for (let line of lines) {
    let rv = line.match(/Card type\s+:\s+(.*?):/);
    if (rv)
      return rv[1];
  }
  return "Generic Camera";
}

// v4l2
function parseSources(dev, lines) {
  const sections = {type:"v4l2", dev, label:getModel(dev), options:[]};
  let currentSection = null;

  // Iterate through each line
  let codec, resolution, fps;
  for (let line of lines) {
    // Check if a new section begins
    const codecMatch = line.match(/^\s*\[(\d+)\]: '(\w+)'/);
    if (codecMatch) {
      const [, sectionIndex, c] = codecMatch;
      codec = c;
    }  

    if (line.trim().startsWith('Size: Discrete')) {
      // Capture size information within the current section
      const rv = line.match(/Size: Discrete ([\d\sx]+)/);
      if (rv) {
        resolution = rv[1];
      }
    }

    if (line.trim().startsWith('Interval: Discrete')) {
      // Capture size information within the current section
      const rv = line.match(/\((\d+\.\d+) fps\)/);
      if (rv) {
        fps = rv[1];
        sections.options.push({resolution, fps, codec});
      }
    }
  }

  return sections;
}


function getSources() {
  let sources = []; // specifically video capture sources

  // Get a list of video devices (e.g. /dev/videoX)
  const listDevicesCmd = 'find /dev -name "video*"';
  const devices = execSync(listDevicesCmd, { encoding: 'utf8' }).split('\n').filter(Boolean);

  devices.forEach(device => {
    const v4l2Cmd = `v4l2-ctl --list-formats-ext -d ${device}`;
    const lines = execSync(v4l2Cmd, { encoding: 'utf8' }).split('\n').filter(Boolean);
    const section = parseSources(device, lines);
    if (section.options.length)
      sources.push(section);
  });
  return sources;
}

module.exports = {
  getSources
};